package io.useless.play.authentication

import java.util.UUID
import scala.util.{ Success, Failure }
import scala.concurrent.Future
import scala.concurrent.ExecutionContext.Implicits.global

import play.api.mvc._

import io.useless.auth.Account
import io.useless.util.Uuid

trait AuthenticatorComponent {

  val authenticator: Authenticator

  trait Authenticator {

    def authenticate[A](request: Request[A]): Future[Option[Account]]

  }

}

trait CompositeAuthenticatorComponent extends AuthenticatorComponent {

  class CompositeAuthenticator(
    authenticators: Seq[Authenticator]
  ) extends Authenticator {

    def authenticate[A](request: Request[A]): Future[Option[Account]] = {
      authenticators.foldLeft(Future.successful(None: Option[Account])) { (futureAccount, authenticator) =>
        futureAccount.filter(_.isDefined).fallbackTo(authenticator.authenticate(request))
      }
    }

  }

}

trait GuidAuthenticatorComponent extends AuthenticatorComponent {

  self: AuthDaoComponent =>

  trait GuidAuthenticator extends Authenticator {

    def guid[A](request: Request[A]): Option[String]

    def authenticate[A](request: Request[A]) = {
      guid(request).map{ rawGuid =>
        Uuid.parseUuid(rawGuid) match {
          case Success(guid: UUID) => {
            authDao.getAccessToken(guid).map { maybeAccessToken =>
              maybeAccessToken.map(_.resourceOwner)
            }
          }
          case _: Failure[UUID] => Future.successful(None)
        }
      }.getOrElse { Future.successful(None) }
    }

  }

}

trait HeaderAuthenticatorComponent extends GuidAuthenticatorComponent {

  self: AuthDaoComponent =>

  class HeaderAuthenticator(header: String) extends GuidAuthenticator {

    def guid[A](request: Request[A]) = request.headers.get(header)

  }

}

trait QueryParameterAuthenticatorComponent extends GuidAuthenticatorComponent {

  self: AuthDaoComponent =>

  class QueryParameterAuthenticator(queryParameter: String) extends GuidAuthenticator {

    def guid[A](request: Request[A]) = request.getQueryString(queryParameter)

  }

}

trait CookieAuthenticatorComponent extends GuidAuthenticatorComponent {

  self: AuthDaoComponent =>

  class CookieAuthenticator(cookie: String) extends GuidAuthenticator {

    def guid[A](request: Request[A]) = request.cookies.get(cookie).map(_.value)

  }

}
